"""
#######################################################################
#
# Licensed Materials - Property of Pipelinefx L.L.C.
#
#
# (C) COPYRIGHT Pipelinefx Limited Liability Corporation.
#  All Rights Reserved.
#                               
#  US Government Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with 
#  Pipelinefx L.L.C.
#
#
# QubeView Subjob module
#
# Name: qbUISubjob.py
#
# $DateTime: 2012/06/12 15:12:47 $
# $Revision: #1 $
# $Change: 9701 $
# $Id: //depot/main/qubeMobileServer/qubeMobile/model/qbUISubjob.py#1 $
# $File: //depot/main/qubeMobileServer/qubeMobile/model/qbUISubjob.py $
#
# Written by: Brian Knudson / PFX
#             briank@pipelinefx.com
#             2011-12-05
#
# Description:
#  Subjob module, subclassed off of qb.Subjob with some extra functions to
#  access logs & human redable time.
#
#
#######################################################################
"""

import os
import sys
import re
import time
import datetime
import mmap
import logging

try:
    import qb
except ImportError:
    api_path = "/Applications/pfx/qube/api/python/"
    if api_path not in sys.path:
        sys.path.insert(0,api_path)
    import qb


class QbUISubjob(qb.Subjob):
    """
    A subjob model.
    Construct with:
     subjob: a qb.Work object
     out_path: path to the owning subjob's stdout cache file
     err_path: path to the owning subjob's stderr cache file
    Note: Both outfp & errfp can be None, but they will never be updated in that case
    """
    def __init__(self,subjob,log_list):
        super(QbUISubjob,self).__init__(subjob)
        self.__log_list = log_list
        


    def getLogs(self,update=False):
        """
        Return the log for this one subjob
        return val is (stdout,stderr)
        """
        logging.debug("Fetching logs for subjob '%s.%s'." % (self['pid'],self['id']))
        return self.__log_list[self['pid']].getSubjobLogs(self['id'])
    

    def getReadableElapsed(self):
        """
        returns the time this subjob has been running in a human
        readable string.
        """
        now = int(time.time())
        start_epoch    = {True:self['timestart'],    False:now}[self['timestart'] > qb.QB_TIME_EPOCH_OFFSET ]
        complete_epoch = {True:self['timecomplete'], False:now}[self['timecomplete'] > qb.QB_TIME_EPOCH_OFFSET ]
        elapsed_epoch = complete_epoch - start_epoch

        return {True:str(datetime.timedelta(seconds=elapsed_epoch)), False:"--" }[elapsed_epoch > 0]


    def getAttribute(self,att):
        """
        returns the attribute from this instance, given an attribute or human
        readable name.
        """
        att = att.lower()
        if self.has_key(att):
            return self[att]
        else:
            if att == 'started':
                return { True:time.strftime("%X",time.localtime(self['timestart'])), False:"--" }[ self['timestart'] > qb.QB_TIME_EPOCH_OFFSET ]
            elif att == 'completed':
                return { True:time.strftime("%X",time.localtime(self['timecomplete'])), False:"--" }[ self['timecomplete'] > qb.QB_TIME_EPOCH_OFFSET ]
            elif att == 'elapsed':
                return self.getReadableElapsed()
            else:
                return att


