"""
#######################################################################
#
# Licensed Materials - Property of Pipelinefx L.L.C.
#
#
# (C) COPYRIGHT Pipelinefx Limited Liability Corporation.
#  All Rights Reserved.
#                               
#  US Government Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with 
#  Pipelinefx L.L.C.
#
# $DateTime: 2012/06/12 15:12:47 $
# $Revision: #1 $
# $Change: 9701 $
# $Id: //depot/main/qubeMobileServer/qubeMobile/model/qbUIUser.py#1 $
# $File: //depot/main/qubeMobileServer/qubeMobile/model/qbUIUser.py $
#
# Written by: Brian Knudson / PFX
#             briank@pipelinefx.com
#             2011-10-05
#
# Description:
#  UIUser class - makes the qb.getuser dict act like any other UI model
#
#
#######################################################################
"""

import os
import logging
from operator import itemgetter

try:
    import qb
except ImportError:
    api_path = "/Applications/pfx/qube/api/python/"
    if api_path not in sys.path:
        sys.path.insert(0,api_path)
    import qb


class QbUIUser(object):

    def __init__(self,username,permissions={}):
        # super(QbUIUser,self).__init__(user)
        self.__username=username
        self.__permissions = self.__expandPermissions(permissions.get('',[]))


    def __expandPermissions(self,permissions):
        """
        sets the permissions list, expanding special items like 'admin' and 'all'
        """
        logging.debug("Setting permissions for '%s'" % self.__username)

        # import pdb; pdb.set_trace()

        try:
            permSet = set(permissions)
        except TypeError:
            logging.error("Cannot set permissions for user '%s'." % self.__username)
            return False
    
        allPermsSet = set([ 
            # Submit permissions 
            "submit_job",
            "submit_callback",
            "submit_global_callback",
            # basic permissions
            "bump",
            "kill",
            "remove",
            "modify",
            "preempt",
            "block",
            "interrupt",
            "complete",
            "unblock",
            "suspend",
            "resume",
            "retry",
            "requeue",
            "migrate",
            "shove",
            "fail",
            "retire",
            "reset",
        ])
    
        adminPermsSet = set([
            # admin permissions
            "lock_host",
            "sudo_admin",
            "impersonate",
            "admin"
        ])


        if 'all' in permSet:
            permSet.update(allPermsSet)
            permSet.discard('all')

        if 'admin' in permSet:
            permSet.update(allPermsSet)
            permSet.update(adminPermsSet)


        return permSet

        

    @property
    def username(self):
        """
        Returns the [read only] username
        """
        return self.__username


    @property
    def permissions(self):
        """
        Returns the [read only] permissions dict
        """
        return self.__permissions


    def hasQubePermission(self, permission=None):
        """
        Determine if a user has a particular qube privilege
    
        return True if the user has the qube 'admin' privilege, as this encompasses all others
        """
        if not permission:
            return True

        return permission in self.__permissions
    

